func.COP.truncated <-
function(y, x.cop, x.cop.covariates = NULL, x.covariates = NULL, h.func = NULL, h.prime.func = NULL, initial.estimate = NULL, 
         group = NULL, exchangeable = FALSE, quasi.lik = FALSE, cop.flag = TRUE, linear.comb=NULL, control=NULL) {
  
  ny = length(y)
  memo = NULL
  cor.exch=NULL
  
  if (class(initial.estimate) ==  "function") {
    init.est = fit.COP.init(y,x.cop,initial.estimate)
  } else {
    init.est = initial.estimate
  }
  
  n.par = length(init.est)
  if (cop.flag) {
      n.comp = 1+length(x.covariates)
      est.labs = c('alpha')
      for (i in 2:n.par) {est.labs = c(est.labs, paste('b[',(i-1),']',sep=''))} 
  } else {
      n.comp = length(x.covariates)
      est.labs = NULL
      for (i in 1:n.par) {est.labs = c(est.labs, paste('b[',i,']',sep=''))} 
  }
  
  model.df = n.par
  
  
  ### initial fit
  
  
  
  parms = list(y=y,x.cop=x.cop,x.covariates=x.covariates, x.cop.covariates=x.cop.covariates, 
               h.func = h.func, h.prime.func = h.prime.func, cop.flag=cop.flag, n = length(y), n.comp=n.comp, model.df=model.df)
 
 
  ### obtain likelihood solution
  
  optim.obj = func.COP.opt(init.est, y=y, parms=parms, control = control)
   
  est = optim.obj$par
  log.lik = optim.obj$value
  names(est) = est.labs
  fitted.values.obj = fit.COP.fitted.values(est,parms)
  
  ### get correlation estimate if needed
  
  if (exchangeable) {cor.exch = func.cor.est(y, fitted.values.obj$mu,length(est),group)[2]}
      
  ### obtain quasi-likelihood solution if requested
  
  if (exchangeable && quasi.lik) {
    
      	quasi1 = rootSolve::multiroot(func.COP.quasi.lik.gradient,start=est,y=y,parms=parms,
      	                              group=group,cor.exch=cor.exch,jacfunc=func.COP.quasi.lik.jacobian)
          
      	### check for convergence 
        	
      	if (sum(is.na(quasi1$f.root))==0) {
      	    est = quasi1$root
      	} else {
      	    memo = c(memo, 'Quasi-likelihood estimate requested but not used')   
      	    quasi.lik = F
      	}
  }      	
  
  ### calculate covariance matrix
  
  if (exchangeable) {
      if (quasi.lik) {
         sigma = func.COP.quasi.lik.cov(est,parms,group,cor.exch)
         sigma = as.matrix(sigma)
      } else {
      		sigma = func.COP.lik.cov.r(est,parms,group,cor.exch)
  		    sigma = as.matrix(sigma)
      }
  } else {
      	sigma = func.COP.lik.cov(est, parms)  
  }
  
  dmu =  fitted.values.obj$dmu  
  fitted.values.se = func.sqrt0(diag(dmu%*%sigma%*%t(dmu)))
  
  coef.est = est
  coef.se = func.sqrt0(diag(sigma))
 
  n.groups=NA 
  if (exchangeable) {n.groups=length(unique(group))}
  
  return(list(coef.est=coef.est,coef.se=coef.se,cor.exch=cor.exch, sigma=sigma))
  
}
